//
//  ViewController.swift
//  Encoder
//
//  Created by Phidgets on 2018-01-29.
//  Copyright © 2018 Phidgets. All rights reserved.
//

import UIKit
import Phidget22Swift

class ViewController: UIViewController {

    let ch = Phidget22Swift.Encoder()
    var phidgetInfoBox:PhidgetInfoBoxViewController! = nil
    var pickerArray = ["pushPull", "lineDriver_2K2", "lineDriver_10K", "openCollector_2K2", "openCollector_10K"]
    
    @IBOutlet var positionChangeLabel: UILabel!
    @IBOutlet var timeChangeLabel: UILabel!
    @IBOutlet var positionLabel: UILabel!
    @IBOutlet var enabledStack: UIStackView!
    @IBOutlet var enabledSwitch: UISwitch!
    @IBOutlet var ioModeButton: UIButton!
    @IBOutlet var dataIntervalSlider: UISlider!
    @IBOutlet var dataIntervalLabel: UILabel!
    @IBOutlet var changeTriggerSlider: UISlider!
    @IBOutlet var changeTriggerLabel: UILabel!
    @IBOutlet var stackView: UIStackView!
    
    override func viewDidLoad() {
        super.viewDidLoad()
        do {
            try Net.enableServerDiscovery(serverType: .deviceRemote)
            let _ = ch.attach.addHandler(attach_handler)
            let _ = ch.detach.addHandler(detach_handler)
            let _ = ch.error.addHandler(error_handler)
            let _ = ch.positionChange.addHandler(positionchange_handler)
            try ch.open()
        } catch let err as PhidgetError {
            outputErrorMessage(errorTitle: "Error Code: \(err.errorCode)", errorMessage: err.description)
        } catch {
            print (error)
        }
    }
    
    //MARK: Event Handlers
    func attach_handler(sender: Phidget){
        do{
            try phidgetInfoBox.fillPhidgetInfo(phid: sender)
            let attachedDevice = sender as! Phidget22Swift.Encoder
            
            let deviceID = try attachedDevice.getDeviceID()
            let minDataInterval = try Float(attachedDevice.getMinDataInterval())
            let maxDataInterval = try Float(attachedDevice.getMaxDataInterval())
            let dataInterval = try Float(attachedDevice.getDataInterval())
            let minPositionChangeTrigger = try Float(attachedDevice.getMinPositionChangeTrigger())
            let maxPositionChangeTrigger = try Float(attachedDevice.getMaxPositionChangeTrigger())
            let positionChangeTrigger = try Float(attachedDevice.getPositionChangeTrigger())
            let isEnabled = try attachedDevice.getEnabled()
            
            DispatchQueue.main.async {
                self.stackView.isHidden = false
                self.dataIntervalSlider.minimumValue = minDataInterval
                self.dataIntervalSlider.maximumValue = maxDataInterval
                self.dataIntervalSlider.value = dataInterval
                self.dataIntervalLabel.text = String(format: "%d ms", UInt32(dataInterval))
                //if min=max interval can not be configured
                if(minDataInterval == maxDataInterval){
                    self.dataIntervalSlider.isEnabled = false
                }
                self.changeTriggerSlider.minimumValue = minPositionChangeTrigger
                self.changeTriggerSlider.maximumValue = maxPositionChangeTrigger
                self.changeTriggerSlider.value = positionChangeTrigger
                self.changeTriggerLabel.text = String(format: "%d", UInt32(positionChangeTrigger))
                
                self.enabledSwitch.isEnabled = isEnabled
                
                
                switch(deviceID){
                case DeviceID.PN_1052:
                    fallthrough
                case DeviceID.PN_1057:
                    fallthrough
                case DeviceID.PN_1065:
                    self.enabledStack.isHidden = true
                    self.ioModeButton.isHidden = true
                    break
                case DeviceID.PN_1047:
                    fallthrough
                case DeviceID.PN_HIN1101:
                    self.enabledStack.isHidden = false
                    self.ioModeButton.isHidden = true
                    break
                case DeviceID.PN_DCC1000:
                    self.enabledStack.isHidden = true
                    self.ioModeButton.isHidden = false
                    break
                case DeviceID.PN_ENC1000:
                    self.ioModeButton.isHidden = false
                    self.enabledStack.isHidden = false
                    break
                default:
                    break
                }
            }

        } catch let err as PhidgetError {
            outputErrorMessage(errorTitle: "Error Code: \(err.errorCode)", errorMessage: err.description)
        } catch {
            print (error)
        }
        
    }
    
    func detach_handler(sender: Phidget){
        do{
            try phidgetInfoBox.fillPhidgetInfo(phid: nil)
            DispatchQueue.main.async {
                self.stackView.isHidden = true
            }
        } catch let err as PhidgetError {
            outputErrorMessage(errorTitle: "Error Code: \(err.errorCode)", errorMessage: err.description)
        } catch {
            print (error)
        }
    }
    
    func error_handler(sender: Phidget, data: (code: ErrorEventCode, description: String)){
        self.outputErrorMessage(errorTitle: "Error Code: \(data.code)", errorMessage: data.description)
    }
    var timeStamp:TimeInterval = 0
    var lastTimeStamp:TimeInterval = 0
    func outputErrorMessage(errorTitle:String, errorMessage:String){
        timeStamp = NSDate.timeIntervalSinceReferenceDate
        //Add a delay for error events so the user does not get bombarded by error events
        if(timeStamp - lastTimeStamp > 5){
            let alert = UIAlertController(title: errorTitle, message: errorMessage, preferredStyle: .alert)
            alert.addAction(UIAlertAction(title: "OK", style: .default, handler: {action -> Void in
            }))
            self.present(alert, animated: true, completion: nil)
            lastTimeStamp = timeStamp
        }
    }
    
    func positionchange_handler(sender: Phidget22Swift.Encoder, data: (positionChange: Int, timeChange: Double, indexTriggered: Bool)){
        do{
            let cumulativePosition = try ch.getPosition()
            
            DispatchQueue.main.async {
                self.positionChangeLabel.text = String(format:"%d ticks", data.positionChange)
                self.timeChangeLabel.text = String(format:"%.3f ms", data.timeChange)
                self.positionLabel.text = String(format: "%ld ticks", cumulativePosition)
            }
        }catch let err as PhidgetError {
            outputErrorMessage(errorTitle: "Error Code: \(err.errorCode)", errorMessage: err.description)
        } catch {
            print (error)
        }
    }
    
    //MARK: GUI controls
    @IBAction func setDataInterval(_ sender: Any) {
        let slider = sender as! UISlider
        do{
            try ch.setDataInterval(UInt32(slider.value))
            DispatchQueue.main.async {
                self.dataIntervalLabel.text = "\(UInt32(slider.value)) ms"
            }
            
        } catch let err as PhidgetError {
            outputErrorMessage(errorTitle: "Error Code: \(err.errorCode)", errorMessage: err.description)
        } catch {
            print (error)
        }
    }
    
    @IBAction func setChangeTrigger(_ sender: Any) {
        let slider = sender as! UISlider
        do{
            try ch.setPositionChangeTrigger(UInt32(slider.value))
            DispatchQueue.main.async {
                self.changeTriggerLabel.text = String(format:"%d ticks", UInt32(slider.value))
            }
            
        } catch let err as PhidgetError {
            outputErrorMessage(errorTitle: "Error Code: \(err.errorCode)", errorMessage: err.description)
        } catch {
            print (error)
        }
    }
    
    @IBAction func setEnabled(_ sender: Any) {
        let enableSwitch = sender as! UISwitch
        do{
            try ch.setEnabled(enableSwitch.isOn)
        } catch let err as PhidgetError {
            outputErrorMessage(errorTitle: "Error Code: \(err.errorCode)", errorMessage: err.description)
        } catch {
            print (error)
        }
    }
    
    //MARK: navigation
    @IBAction func pickerExit(_ segue: UIStoryboardSegue){
        dimBackground()
    }
    
    override func prepare(for segue: UIStoryboardSegue, sender: Any?) {
        if segue.identifier == "infoBoxSegue" {
            phidgetInfoBox = segue.destination as! PhidgetInfoBoxViewController
        }
        else if segue.identifier == "ioModeSegue"{
            let vc:PickerViewController = segue.destination as! PickerViewController
            dimBackground()
            vc.titleName = "Sensor Type"
            vc.pickerArray = pickerArray
            vc.ch = ch
        }
    }
    
    //MARK: Misc
    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
    }
    
    func dimBackground(){
        self.view.alpha = (self.view.alpha == 1.0) ? 0.5 : 1.0
    }


}

